#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser <oli.sennhauser@bluewin.ch>.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

# Parameters
# ----------

my $pErrorLog = '/var/mysql/data/linux.err';
my $pDatabaseName = 'defaultDB';
my $pConfigurationFile = '/tmp/check_errorlog_mysql.conf';

my $rc = GetOptions(
    'error-log|e=s' => \$pErrorLog
  , 'database|d=s' => \$pDatabaseName
  , 'configuration|c=s' => \$pConfigurationFile
);

if ( ! $rc ) {
  print("Error in parameters.");
  exit($lError{'Unknown'});
}

if( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------
my $lConfigurationFileTmp = "$pConfigurationFile.tmp";

my @lFilterConfiguration = (
    '^Version: \'.*\'  socket: \'.*\'  port: \d*\s*.*$'
  , '^Number of processes running now: \d*$'
  , '^.*: ready for connections.?$'
  , '^InnoDB: a new database to be created!$'
  , '^InnoDB: Creating foreign key constraint system tables$'
  , '^InnoDB: Database physically writes the file full: wait...$'
  , '^InnoDB: Doublewrite buffer created$'
  , '^InnoDB: Doublewrite buffer not found: creating new$'
  , '^InnoDB: Foreign key constraint system tables created$'
  , '^InnoDB: Setting log file .* size to \d* MB$'
  , '^InnoDB: The first specified data file .* did not exist:$'
  , '^InnoDB: Reading tablespace information from the .ibd files...$'
  , '^InnoDB: buffer...$'
  , '^InnoDB: log sequence number \d* \d*.$'
  , '^InnoDB: Progress in percents:(\s\d*)*$'
  , '^InnoDB: Apply batch completed$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave SQL thread initialized, starting replication in log \'.*\' at position \d*, relay log \'.*\' position: \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave I/O thread killed while reading event$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave I/O thread: connected to master \'.*\',  replication started in log \'.*\' at position \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave I/O thread killed while reading event$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] .*: ready for connections.$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave I/O thread exiting, read up to log \'.*\', position \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] .*: Shutdown complete$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] .*: Normal shutdown$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave: connected to master \'.*\',replication resumed in log \'.*\' at position \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave SQL thread exiting, replication stopped in log \'.*\' at position \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Log file .* did not exist: new to be created$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Setting file .* size to \d* MB$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Shutdown completed; log sequence number \d*\s*\d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Started; log sequence number \d*\s*\d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Started$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Starting an apply batch of log records to the database...$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Starting log scan based on checkpoint at$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Starting shutdown...$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Shutdown completed$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*mysqld (ended|started)$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*.*: Shutdown Complete$'
);

# Read last position form configuration file
# ------------------------------------------
my $lOldTs = 0;

if ( -r $pConfigurationFile ) {
  open(CONF, '<' . $pConfigurationFile);
  while ( <CONF> ) {
    chomp;
    if ( m/^$pDatabaseName:(\d*)$/ ) {
      $lOldTs = $1;
      last;
    }
  }
  close(CONF);
}
else {
  `touch $pConfigurationFile`;
}

# Parse the error log
# -------------------

my $lFilter;
my $lTs;
my $f;
my $lNewTs = 0;
my $lErrorCounter = 0;
my $lWarningCounter = 0;

if ( ! -r $pErrorLog ) {
  print("No mysql error log found on $pErrorLog. Please check configuration!");
  exit($lError{'Warning'});
}

open(LOG, '<' . $pErrorLog);
my $lInTime = 0;
while ( <LOG> ) {

  # skip empty lines
  if ( m/^$/ ) {
    next;
  }

  if ( (! $lInTime) and m/^(\d{6}\s*\d{1,2}:\d{2}:\d{2})/ ) {
     $lTs = $1;
     $lTs =~  s/[ :]//g;
     # potential loss of records within the same second!!!
     if ( $lTs <= $lOldTs ) {
       next;
     }
     else {
       $lInTime = 1;
     }
  }

  if ($lInTime) {

    $f = 0;
    foreach $lFilter (@lFilterConfiguration) {

      if ( m/$lFilter/ ) {
        $f++;
        last;
      }
    }
    if ( $f ) {
      next;
    }

    print $_;
    if ( m/error/i ) {
      $lErrorCounter++;
    }
    elsif ( m/warning/i ) {
      $lWarningCounter++;
    }

    if ( m/^(\d{6}\s*\d{1,2}:\d{2}:\d{2})/ ) {
      $lNewTs = $1;
      $lNewTs =~ s/[ :]//g;
    }
  }
}
close(LOG);

$lNewTs =~ s/^0*//;

if ( $lNewTs eq '' ) {
  $lNewTs = $lOldTs;
}

# Set last position
# -----------------
open(CONF, '<' . $pConfigurationFile);
open(TMP, '>' . $lConfigurationFileTmp);
while ( <CONF> ) {
  if ( ! m/^$pDatabaseName:(\d*)$/ ) {
    print(TMP $_);
  }
}
print(TMP "$pDatabaseName:$lNewTs\n");
close(TMP);
close(CONF);
`mv $lConfigurationFileTmp $pConfigurationFile`;

if ( $lErrorCounter ) {
  exit($lError{'Critical'});
}
elsif ( $lWarningCounter ) {
  exit($lError{'Warning'});
}

print("No errors or warnings found.");
exit($lError{'OK'});
