#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser <oli.sennhauser@bluewin.ch>.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

# Parameters
# ----------

#CREATE TABLE test.heartbeat (
#    id   INT
#  , beat INT
#);

my $pMasterUser = '';
my $pMasterPassword = '';
my $pMasterHost = 'localhost';
my $pMasterPort = '3306';
my $pSlaveUser = 'replication';
my $pSlavePassword = '';
my $pSlaveHost = 'localhost';
my $pSlavePort = '3306';
my $pHeartbeatSchema = 'test';
my $pHeartbeatTable = 'heartbeat';
my $pHeartbeatBeatField = 'beat';
my $pHeartbeatIdField = 'id';
my $pHeartbeatId = 1;
my $pWarning = 4;
my $pCritical = 25;

my $rc = GetOptions(
    'master-user=s' => \$pMasterUser
  , 'master-password=s' => \$pMasterPassword
  , 'master-host=s' => \$pMasterHost
  , 'master-port=i' => \$pMasterPort
  , 'slave-user=s' => \$pSlaveUser
  , 'slave-password=s' => \$pSlavePassword
  , 'slave-host=s' => \$pSlaveHost
  , 'slave-port=i' => \$pSlavePort
  , 'warning|w=i' => \$pWarning
  , 'critical|c=i' => \$pCritical
);

if ( ! $rc ) {
  print("Error in parameters. User = $pMasterUser, Password=hidden, Host = $pMasterHost, Port = $pMasterPort");
  exit($lError{'Unknown'});
}

if( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------

my ($dbhM, $dbhS, $sql, $sth);

$dbhM = DBI->connect("DBI:mysql::$pMasterHost:$pMasterPort"
                   , $pMasterUser, $pMasterPassword
                   , { RaiseError => 0
                     , PrintError => 0
                     , AutoCommit => 0
                     }
                    );

if ( DBI::err ) {

  if ( $DBI::errstr =~ m/Can't connto to/ ) {
    print("Error during connection");
    exit($lError{'Critical'});
  }

  if ( $DBI::errstr =~ m/Access denied for user/ ) {
    print("User $pMasterUser does not have access privileges to database.");
    exit($lError{'Warning'});
  }

  print("Error during connection: " . $DBI::errstr );
  exit($lError{'Critical'});
}

$dbhS = DBI->connect("DBI:mysql::$pSlaveHost:$pSlavePort"
                   , $pSlaveUser, $pSlavePassword
                   , { RaiseError => 0
                     , PrintError => 0
                     , AutoCommit => 0
                     }
                    );

if ( DBI::err ) {

  if ( $DBI::errstr =~ m/Can't connto to/ ) {
    print("Error during connection");
    exit($lError{'Critical'});
  }

  if ( $DBI::errstr =~ m/Access denied for user/ ) {
    print("User does not have access privileges to database.");
    exit($lError{'Warning'});
  }

  print("Error during connection: " . $DBI::errstr );
  exit($lError{'Critical'});
}

$sql = "
SELECT $pHeartbeatBeatField
  FROM $pHeartbeatSchema.$pHeartbeatTable
 WHERE $pHeartbeatIdField = ?
";

$sth = $dbhM->prepare( $sql );
if ( $DBI::err ) {
  print("Error in preparing");
  $dbhM->disconnect;
  exit($lError{'Critical'});
}

$sth->execute($pHeartbeatId);
if ( $sth->err ) {
  print("Error in executing: " . $sth->errstr );
  $dbhM->disconnect;
  exit($lError{'Critical'});
}

my ($lMasterBeat);
$sth->bind_columns(undef, \$lMasterBeat);
if ( $sth->err ) {
  print("Error in binding");
  $sth->finish;
  $dbhM->disconnect;
  exit($lError{'Critical'});
}

$sth->fetchrow_arrayref();
if ( $sth->err ) {
  print("Error in fetchting");
  $sth->finish;
  $dbhM->disconnect;
  exit($lError{'Critical'});
}
$sth->finish;

# ----------------------------------------------------------------------

$sth = $dbhS->prepare( $sql );
if ( $DBI::err ) {
  print("Error in preparing");
  $dbhS->disconnect;
  exit($lError{'Critical'});
}

$sth->execute($pHeartbeatId);
if ( $sth->err ) {
  print("Error in executing: " . $sth->errstr );
  $dbhS->disconnect;
  exit($lError{'Critical'});
}

my ($lSlaveBeat);
$sth->bind_columns(undef, \$lSlaveBeat);
if ( $sth->err ) {
  print("Error in binding");
  $sth->finish;
  $dbhS->disconnect;
  exit($lError{'Critical'});
}

$sth->fetchrow_arrayref();
if ( $sth->err ) {
  print("Error in fetchting");
  $sth->finish;
  $dbhS->disconnect;
  exit($lError{'Critical'});
}
$sth->finish;

$dbhM->disconnect;
$dbhS->disconnect;

if (! defined($lMasterBeat) ) {
  print("Heartbeat is not yet setup properly on master");
  exit($lError{'Warning'});
}

if (! defined($lSlaveBeat) ) {
  print("Heartbeat is not yet setup properly on slave");
  exit($lError{'Warning'});
}

my $lBeatDelta = $lMasterBeat - $lSlaveBeat;

if ( $lBeatDelta > $pCritical ) {
  print("Slave is $lBeatDelta beats behind master which is more than the allowed CRITICAL level of $pCritical beats");
  exit($lError{'Critical'});
}
elsif ( $lBeatDelta > $pWarning ) {
  print("Slave is $lBeatDelta beats behind master which is more than the allowed WARNING level of $pWarning beats");
  exit($lError{'Warning'});
}

print("Heartbeat looks fine, Slave is $lBeatDelta beats behind master");
exit($lError{'OK'});
